# EPICS IOC Startup Script generated by https://github.com/epics-containers/ibek

cd "/repos/epics/ioc"

epicsEnvSet EPICS_CA_MAX_ARRAY_BYTES 6000000
epicsEnvSet EPICS_CA_SERVER_PORT 7064

dbLoadDatabase dbd/ioc.dbd
ioc_registerRecordDeviceDriver pdbbase

dbLoadRecords("config/ioc.db")

dbLoadRecords /tmp/ioc.db
iocInit

